/*
 * Copyright (c) 2024 ShinGeTsu Meter.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include "hardware/gpio.h"
#include "hardware/i2c.h"

#include "aqm1602.h"
#include "mcp3424.h"
#include "mcp4726.h"

#include "i2cDev.h"

/**
 * @brief I2Cデバイスを初期化
 */
void initI2C() {
    // for LCD
    i2c_init(i2c0, I2C0_SPEED_kHz * 1000);
    gpio_set_function(I2C0_SDA_PIN, GPIO_FUNC_I2C);
    gpio_set_function(I2C0_SCL_PIN, GPIO_FUNC_I2C);
    gpio_pull_up(I2C0_SDA_PIN);
    gpio_pull_up(I2C0_SCL_PIN);
    LCD_Init(i2c0, ADDR_LCD);

    // for ACD,DAC
    i2c_init(i2c1, I2C1_SPEED_kHz * 1000);
    gpio_set_function(I2C1_SDA_PIN, GPIO_FUNC_I2C);
    gpio_set_function(I2C1_SCL_PIN, GPIO_FUNC_I2C);
    gpio_pull_up(I2C1_SDA_PIN);
    gpio_pull_up(I2C1_SCL_PIN);

    MCP4726_Init(i2c1, ADDR_MCP4726);
    MCP4726_Set(0);
    MCP3424_Init(i2c1, ADDR_MCP3424);
}

// ADC /////////////////////////////////////////////////

static double readV(bool highReso, uint8_t ch) {
    MCP3424_startConversion(ch, highReso ? BIT_14 : BIT_12, true);
    double result = MCP3424_read();
    return result;
}

/** @brief 定電圧値を計測する */
double readConstV(bool highReso) { return readV(highReso, CH1); }

/**  @brief DACの電圧値を計測する */
double readDacV(bool highReso) { return readV(highReso, CH2); }

/** @brief PCの電圧値を計測する */
double readPcV(bool highReso) { return readV(highReso, CH3); }

/** @brief メーターへの出力電圧値を集計する */
double readOutputV(bool highReso) { return (readV(highReso, CH4) * OUTPUT_DIV); }

// DAC /////////////////////////////////////////////////

int16_t setDacValue(int16_t value) {
    if (value < DAC_MIN_VALUE) {
        value = DAC_MIN_VALUE;
    } else if (value > DAC_MAX_VALUE) {
        value = DAC_MAX_VALUE;
    }

    MCP4726_Set(value);
    sleep_us(DAC_WAIT_US);
    return value;
}

// LCD /////////////////////////////////////////////////

/**
 * @brief LCDへメッセージを表示する
 */
void lcdPrint(uint8_t row, char *message) {
    LCD_SelectRow(row);
    LCD_Write(message);
}

/**
 * @brief LCDをクリアする
 */
void lcdClear() {
    LCD_Clear();
}